package data;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.StreamTokenizer;

public class Data {
	private int matriz[][];
	private int demanda[];
	private boolean cidStatus[];
	private int nCidades;
	private int nCaminhoes;
	private int[] caminhoes;
	private int[] custos;
	private static Data instance = null;
	
	public static String citiesFile = "arquivos/vrp8.txt";
	public static String trucksFile = "arquivos/caminhoes.txt";
	private static int peso = 10000;
    private static int vizinhos = 15;
    private static int btMax = 150;
    private static double alfa = 0.7;
    private static int lBtSize = 10;
    private static int graspMax = 100;
    
	private Data(FileReader cities, FileReader trucks) {
		readCitiesData(cities);
		readTrucksData(trucks);
		cidStatus = new boolean[nCidades];
		clearStatus();
	}

	private void readCitiesData(FileReader is) {
		try {
			StreamTokenizer st = new StreamTokenizer(is);
			st.nextToken();
			nCidades = (int) st.nval;
			nCidades++; // incluindo depsito de coordenada (0,0)
			int cidades[][] = new int[nCidades][2];
			demanda = new int[nCidades - 1];
			matriz = new int[nCidades][nCidades];
			int linha = 0, coluna = 0;
			int k = 0;
			while (st.nextToken() != StreamTokenizer.TT_EOF) {
				linha = (int) st.nval;
				for (int i = 0; i < 2; i++) { // lendo os pontos
					st.nextToken();
					cidades[linha][i] = (int) st.nval;
				}
				st.nextToken();
				if(k > 0) demanda[linha-1] = (int) st.nval;
				else k++;
			}
			calcula_distancias(cidades);
			nCidades--; // excluindo o depsito
		} catch (IOException io) {
			io.printStackTrace();
		}
	}

	private void calcula_distancias(int[][] cidades) {
		for (int i = 0; i < nCidades; i++) {
			for (int j = 0; j < nCidades; j++) {
				matriz[i][j] = 0;
				if (i == j) {
					//matriz[i][j] = Integer.MAX_VALUE;
					continue;
				}
				double dx = Math.pow((cidades[j][0] - cidades[i][0]), 2);
				double dy = Math.pow((cidades[j][1] - cidades[i][1]), 2);
				matriz[i][j] = (int) Math.sqrt(dx + dy);
			}
		}
	}

	private void readTrucksData(FileReader fr) {
		try {
			StreamTokenizer st = new StreamTokenizer(fr);
			st.nextToken();
			nCaminhoes = (int) st.nval;
			caminhoes = new int[nCaminhoes];
			custos = new int[nCaminhoes];
			int linha;
			while (st.nextToken() != StreamTokenizer.TT_EOF) {
				linha = (int) st.nval;
				st.nextToken();
				caminhoes[linha] = (int) st.nval;
				st.nextToken();
				custos[linha] = (int) st.nval;
			}
		} catch (IOException io) {
			io.printStackTrace();
		}
	}

	public int getCapacityOfTruck(int t) {
		return caminhoes[t];
	}

	public int getCostOfTruck(int t) {
		return custos[t];
	}

	public int getDemanda(int c) {
		return demanda[c-1];
	}

	public int getDistancia(int origem, int fim) {
		return matriz[origem][fim];
	}

	public static Data getPRVData() {
		if (instance == null)
			try {
				instance = new Data(
					new FileReader(new File(citiesFile)),
					new FileReader(new File(trucksFile)));
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		return instance;
		
	}

	public void clearStatus() {
		for (int i = 0; i < nCidades; i++) {
			cidStatus[i] = false;
		}
	}

	public boolean isVisited(int cid) {
		if(cid<1 || cid >=nCidades) return true;
		return cidStatus[cid-1];
	}

	public void setVisit(int cid) {
		cidStatus[cid-1] = true;
	}

	public int[] getNotVisited() {
		int l[] = new int[nCidades - getNumOfVisited()];
		int c = 0;
		for (int i = 0; i < nCidades; i++) {
			if (!cidStatus[i]) {
				l[c] = i+1;
                c++;
			}
		}
		return l;
	}

	public int getNumOfVisited() {
		int n = 0;
		for (int i = 0; i < nCidades; i++) {
			if (cidStatus[i])
				n++;
		}
		return n;
	}

	public int getNCaminhoes() {
		return nCaminhoes;
	}

	public int getNCidades() {
		return nCidades;
	}

	public static void setPeso(int p) {
		peso = p;
	}

	public int getPeso() {
		return peso;
	}

	public static int random(int n) {
		return (int) (Math.random() * n);
	}

	public static int random(int n1, int n2) {
		return (int) (Math.random() * (n2 - n1)) + n1;
	}
	
	public void printData() {
		for(int i = 0; i < matriz.length;i++){
		   for(int j = 0; j < matriz[i].length;j++)
		      System.out.println((i) + " - > " + (j)+ " " + matriz[i][j] + " ");
		}     
	}

	/**
	 * @return
	 */
	public static int getVizinhos() {
		return vizinhos;
	}

	/**
	 * @param i
	 */
	public static void setVizinhos(int i) {
		vizinhos = i;
	}

	/**
	 * @return
	 */
	public static double getAlfa() {
		return alfa;
	}

	/**
	 * @return
	 */
	public static int getBtMax() {
		return btMax;
	}

	/**
	 * @return
	 */
	public static int getLBtSize() {
		return lBtSize;
	}

	/**
	 * @return
	 */
	public static int getGraspMax() {
		return graspMax;
	}

	/**
	 * @param i
	 */
	public static void setGraspMax(int i) {
		graspMax = i;
	}

	/**
	 * @param i
	 */
	public static void setLBtSize(int i) {
		lBtSize = i;
	}

	/**
	 * @param i
	 */
	public static void setBtMax(int i) {
		btMax = i;
	}

}
